@@ -21,6 +21,7 @@ import hashlib
 import hmac
 import json
 import logging
+import mailbox
 import os
 import shutil
 from subprocess import call
@@ -213,7 +214,15 @@ I have just signed your key
       $fingerprint
 
 
+The certification is in an attachment encrypted to your key.
+You can either drag the attachment into GNOME Keysign or
+decrypt and import the attachment manually, e.g.
+
+    gpg --decrypt  |  gpg --import
+
+
 Thanks for letting me sign your key!
+And, if you can, send me copy of your key with the new certifications.
 
 --
 GNOME Keysign
@@ -243,8 +252,9 @@ def sign_keydata_and_send(keydata, error_cb=None):
     For the resulting signatures, emails are created and
     sent via send_email.
     
-    Return value:  NamedTemporaryFiles used for saving the signatures.
-    If you let them go out of scope they should get deleted.
+    Return value:  A tuple of a
+    NamedTemporaryFiles used for saving the signatures and the plaintext which got encrypted.
+    If you let the TemporaryFiles go out of scope they should get deleted.
     But don't delete too early as the MUA needs to pick them up.
     """
     log = logging.getLogger(__name__ + ':sign_keydata')
@@ -261,7 +271,7 @@ def sign_keydata_and_send(keydata, error_cb=None):
     except AttributeError:
         log.debug("keydata is probably already a bytes type")
 
-    for uid, encrypted_key in list(sign_keydata_and_encrypt(keydata, error_cb)):
+    for uid, encrypted_key, plaintext in list(sign_keydata_and_encrypt(keydata, error_cb)):
         log.info("Using UID: %r", uid)
         # We expect uid.uid to be a consumable string
         uid_str = uid.uid
@@ -291,7 +301,7 @@ def sign_keydata_and_send(keydata, error_cb=None):
         subject = Template(SUBJECT).safe_substitute(ctx)
         body = Template(body).safe_substitute(ctx)
         send_email(uid.email, subject, body, [filename])
-        yield tmpfile
+        yield tmpfile, plaintext
 
 
 def format_fingerprint(fpr):
@@ -357,16 +367,18 @@ def strip_fingerprint(input_string):
 
 
 def download_key_http(address, port):
+    netloc = "[%s]:%d" if ':' in address else "%s:%d"
     url = ParseResult(
         scheme='http',
         # This seems to work well enough with both IPv6 and IPv4
-        netloc="[[%s]]:%d" % (address, port),
+        netloc=netloc % (address, port),
         path='/',
         params='',
         query='',
         fragment='')
-    log.debug("Starting HTTP request")
-    data = requests.get(url.geturl(), timeout=5).content
+    u = url.geturl()
+    log.debug("Starting HTTP request for %s", u)
+    data = requests.get(u, timeout=5).content
     log.debug("finished downloading %d bytes", len(data))
     return data
 
@@ -455,3 +467,14 @@ def _get_available_bt():
             bt = '/'.join((object_path, child.attrib['name']))
             available_bt.append(bt)
     return available_bt
+
+
+def get_attachments(filename):
+    mbox = mailbox.mbox(filename)
+    attachments = []
+    for message in mbox:
+        # Check if there are attachments
+        if message.is_multipart():
+            for attach in message.get_payload()[1:]:
+                attachments.append(attach.get_payload(decode=True))
+    return attachments
